"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Methods = require("./methods");
/**
 * AsyncArray Class
 *
 * @class AsyncArray
 * @extends {Array<T>}
 * @template T
 */
class AsyncArray extends Array {
    /**
     * Creates an instance of AsyncArray
     * @param {...T[]} args
     * @memberof AsyncArray
     */
    constructor(...args) {
        super(...args);
        this.input = args;
    }
    /**
     * Async Every method
     *
     * @param {Methods.CallBackFilter<T>} cb
     * @returns {Promise<boolean>}
     * @memberof AsyncArray
     */
    aEvery(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            return Methods.every(this.input, cb);
        });
    }
    /**
     * Async Filter method
     *
     * @param {Methods.CallBackFilter<T>} cb
     * @returns {Promise<T[]>}
     * @memberof AsyncArray
     */
    aFilter(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            return Methods.filter(this.input, cb);
        });
    }
    /**
     * Async find method
     *
     * @param {Methods.CallBackFind<T>} cb
     * @returns {(Promise<T | undefined>)}
     * @memberof AsyncArray
     */
    aFind(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            return Methods.find(this.input, cb);
        });
    }
    /**
     * Async ForEach method
     *
     * @param {Methods.CallBackForEach<T>} cb
     * @returns {Promise<void>}
     * @memberof AsyncArray
     */
    aForEach(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Methods.forEach(this.input, cb);
        });
    }
    /**
     * Async Map method
     *
     * @template R
     * @param {Methods.CallBackMap<T, R>} cb
     * @returns {Promise<R[]>}
     * @memberof AsyncArray
     */
    aMap(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            return Methods.map(this.input, cb);
        });
    }
    /**
     * Async Reduce method
     *
     * @template R
     * @param {Methods.CallBackReduce<T, R>} cb
     * @param {R} [initialValue]
     * @returns {Promise<R>}
     * @memberof AsyncArray
     */
    aReduce(cb, initialValue) {
        return __awaiter(this, void 0, void 0, function* () {
            return Methods.reduce(this.input, cb, initialValue);
        });
    }
    /**
     * Async Some method
     *
     * @param {Methods.CallBackFilter<T>} cb
     * @returns {Promise<boolean>}
     * @memberof AsyncArray
     */
    aSome(cb) {
        return __awaiter(this, void 0, void 0, function* () {
            return Methods.some(this.input, cb);
        });
    }
}
exports.AsyncArray = AsyncArray;
//# sourceMappingURL=async_ray.js.map